// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/id/perform.c $
// $Rev: 52926 $
// $Date: 2023-04-20 15:42:21 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _input_modes(void)
{
	gsc_label("Input Modes");
	printf("Differential, Single-Ended, Zero Test, +Vref Test, Chan 0/1 Loopback\n");
}



//*****************************************************************************
static void _voltage_ranges(void)
{
	gsc_label("Voltage Ranges");
	printf("+-10V, +-5V, +-2.5V\n");
}



//*****************************************************************************
static void _clocking_sources(void)
{
	gsc_label("Clocking Sources");
	printf("Rate Generator(s), External, Software\n");
}



//*****************************************************************************
static void _sampling_modes(void)
{
	gsc_label("Input Sampling Modes");
	printf("Continuous, Burst\n");

	gsc_label("Output Sampling Modes");
	printf("Continuous, Burst\n");

	gsc_label("Output Update Modes");
	printf("Immediate, Synchronous\n");

	gsc_label("Output Modes");
	printf("Buffered, Unbuffered\n");

	gsc_label("Output Buffer Modes");
	printf("Open, Closed (recycle)\n");
}



//*****************************************************************************
static void _digital_io(void)
{
	gsc_label("Digital I/O");
	printf("6 Bi-Directional Signals\n");
}



//*****************************************************************************
static int _id_device_acr(const args_t* args)
{
	gsc_reg_def_t	list[2]	= { { NULL }, { NULL } };
	gsc_reg_def_t*	ptr;

	int	errs	= 0;

	printf("\n");
	ptr	= (gsc_reg_def_t*) aiss16ao2_reg_get_def_id(AISS16AO2_GSC_ACR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(args->fd, list, 1, aiss16ao2_reg_read);
	}
	else
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_reg_get_def_id, AISS16AO2_GSC_ACR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
static int _register_map(const args_t* args)
{
	int	errs;

	printf("\n");
	errs	= aiss16ao2_reg_list(args->fd, args->detail);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;

	gsc_label("Device Features");
	printf("\n");

	gsc_label_level_inc();

	_input_modes();
	_voltage_ranges();
	_clocking_sources();
	_sampling_modes();
	_digital_io();

	for (i = 0; i < AISS16AO2_IOCTL_QUERY_LAST; i++)
		errs	+= aiss16ao2_query(args->fd, -1, 1, i, NULL);

	gsc_label_level_dec();

	errs	+= _id_device_acr(args);
	errs	+= _register_map(args);
	printf("\n");

	return(errs);
}


