// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/rxrate/perform.c $
// $Rev: 52927 $
// $Date: 2023-04-20 15:42:48 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1 * 1024 * 1024)	// size of driver's transfer buffer



// variables ******************************************************************

static	u8	_rx[_1M];



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= aiss16ao2_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if (total < limit)
			continue;

		if (now.tv_sec < minimum.tv_sec)
			continue;

		if ((now.tv_sec == minimum.tv_sec) && (now.tv_usec < minimum.tv_usec))
			continue;

		us	= (now.tv_sec - begin.tv_sec) * 1000000L
			+ (now.tv_usec - begin.tv_usec);
		sps	= (total * 10000000L / 4 / us + 5) / 10;

		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
		gsc_label_long_comma(sps);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs			= 0;
	long	get				= sizeof(_rx);
	s32		master_clock	= args->master_clock;

	if (master_clock < 0)
		errs	+= aiss16ao2_query		(args->fd, -1, 0, AISS16AO2_QUERY_MASTER_CLOCK, &master_clock);

	errs	+= aiss16ao2_config_ai		(args->fd, -1, 1, 1000000L, master_clock);
	errs	+= aiss16ao2_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= aiss16ao2_rx_io_overflow	(args->fd, -1, 1, AISS16AO2_IO_OVERFLOW_IGNORE, NULL);
	errs	+= aiss16ao2_rx_io_underflow(args->fd, -1, 1, AISS16AO2_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %d MB minimum)\n", args->seconds, args->rx_mb);
	gsc_label_level_inc();

	errs	+= aiss16ao2_ai_buf_clear		(args->fd, -1, 1);
	errs	+= aiss16ao2_ai_buf_overflow	(args->fd, -1, 1, AISS16AO2_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= aiss16ao2_ai_buf_underflow	(args->fd, -1, 1, AISS16AO2_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, get);

	errs	+= aiss16ao2_ai_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= aiss16ao2_ai_buf_underflow	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



