// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/savedata/perform.c $
// $Rev: 52928 $
// $Date: 2023-04-20 15:43:09 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M				(1024L * 1024L)
#define	TAG_DECODE(v)	((v) & 0x10000)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	int	errs;

	gsc_label("Input Channels");
	errs	=  aiss16ao2_query(args->fd, -1, 0, AISS16AO2_QUERY_CHANNEL_AI_QTY, chans);

	if (errs == 0)
		printf("%ld Channels\n", (long) chans[0]);

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	char	buf[80];
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	int		got;
	int		index;

	for (index = 0; (errs == 0) && (index < args->repeat);)
	{
		index++;
		strcpy(buf, "Reading");

		if (args->repeat > 1)
		{
			strcat(buf, " #");
			gsc_label_long_comma_buf(index, buf + strlen(buf));
		}

		gsc_label(buf);

		got	= aiss16ao2_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <---  (read error: %d)\n", got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
					(long) got,
					(long) get);
			break;
		}

		errs	= 0;
		printf("PASS  (");
		gsc_label_long_comma(get / 4);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _save_eol(const char* name, FILE* file)
{
	int	errs	= 0;
	int	i;

	i	= fprintf(file, "\r\n");

	if (i != 2)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_value(const args_t* args, const char* name, FILE* file, u32 value)
{
	char	buf1[40];
	char	buf2[80];
	int		errs	= 0;
	int		i;
	int		len;

	if (args->tag_strip)
		value &= 0xFFFF;

	if (args->decimal)
		sprintf(buf1, "%ld", (long) value);
	else
		sprintf(buf1, "%08lX", (long) value);

	sprintf(buf2, "  %8s", buf1);
	len	= strlen(buf2);
	i	= fprintf(file, "%8s", buf2);

	if (i != len)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _validate_data(int chans, u32 last, long index, u32 data)
{
	static	int	chan_tag	= 0;
	int			errs		= 0;
	int			expect;
	static	int	last_tag	= 0;

	if (data & 0xFFFC0000)
	{
		errs	= 1;
		printf(" FAIL <---  (Upper bits are invalid: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	chan_tag	= TAG_DECODE(data) ? 0 : chan_tag + 1;

	if (chan_tag >= chans)
	{
		errs	= 1;
		printf(" FAIL <---  (Invalid Channel Tag: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	if (last == 0xFFFFFFFF)
	{
		if (chan_tag != 0)
		{
			errs	= 1;
			printf(" FAIL <---  (First tag not zero: [%ld] = 0x%08lX)\n", index, (long) data);
		}
	}
	else
	{
		last_tag	= TAG_DECODE(last) ? 0 : last_tag + 1;
		expect		= (last_tag + 1) % chans;

		if (chan_tag != expect)
		{
			errs	= 1;
			printf(	" FAIL <---  (Lost Synchronozation: [%ld] = 0x%08lX, [%ld] = 0x%08lX)\n",
					index - 1,
					(long) last,
					index,
					(long) data);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(const args_t* args, int chans, int errs)
{
	char		buf[64];
	FILE*		file;
	long		l;
	u32			last		= 0xFFFFFFFF;
	const char*	name		= "data.txt";
	long		samples		= sizeof(_buffer) / 4;
	int			tag_got;

	strcpy(buf, "Saving");

	if (args->force_save)
	{
		errs	= 0;
		strcat(buf, " (Forced)");
	}

	gsc_label(buf);

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			tag_got	= TAG_DECODE(_buffer[l]) ? 0 : 1;

			if ((l > 0) && (tag_got == 0))
				errs	+= _save_eol(name, file);

			errs	+= _save_value(args, name, file, _buffer[l]);
			errs	+= _validate_data(chans, last, l, _buffer[l]);
			last	= _buffer[l];
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 16;
	int	errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aiss16ao2_config_ai			(args->fd, -1, 1, args->fsamp, args->master_clock);
	errs	+= aiss16ao2_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels(args, &chans);
	errs	+= aiss16ao2_rx_io_overflow		(args->fd, -1, 1, AISS16AO2_IO_OVERFLOW_IGNORE, NULL);
	errs	+= aiss16ao2_rx_io_underflow	(args->fd, -1, 1, AISS16AO2_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= aiss16ao2_ai_buf_clear		(args->fd, -1, 1);
	errs	+= aiss16ao2_ai_buf_overflow	(args->fd, -1, 1, AISS16AO2_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= aiss16ao2_ai_buf_underflow	(args->fd, -1, 1, AISS16AO2_BUF_UNDERFLOW_CLEAR, NULL);
	errs	+= aiss16ao2_rx_io_timeout		(args->fd, -1, 1, 300, NULL);
	errs	+= _read_data(args);
	errs	+= aiss16ao2_ai_buf_overflow	(args->fd, -1, 1, AISS16AO2_BUF_OVERFLOW_TEST, NULL);
	errs	+= aiss16ao2_ai_buf_underflow	(args->fd, -1, 1, AISS16AO2_BUF_UNDERFLOW_TEST, NULL);
	errs	+= _save_data(args, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



