// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ai_buf_underflow.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	int errs	= 0;

	// There are no persistent bits to test with this service.
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AIBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that there is an underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x8,
			/* value	*/	0x8
		},
		{					// Verify that TEST changes nothing.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_UNDERFLOW,
			/* arg		*/	AISS16AO2_BUF_UNDERFLOW_TEST,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x8,
			/* value	*/	0x8
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{					// Verify that clear REMOVES THE STATUS.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_UNDERFLOW,
			/* arg		*/	AISS16AO2_BUF_UNDERFLOW_CLEAR,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x8,
			/* value	*/	0x0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts		= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	// Verify that the status is NO underflow.

	if (errs == 0)
	{
		sts	= -1;
		ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AI_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AISS16AO2_BUF_UNDERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. Underflow occurred but was unexpected.)\n",
					__LINE__);
		}
	}

	// Verify that the status IS an underflow.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		sts	= -1;
		ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AI_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AISS16AO2_BUF_UNDERFLOW_YES)
		{
			errs++;
			printf(	"FAIL <---  (%d. Underflow expected but did not occur.)\n",
					__LINE__);
		}
	}

	// Verify that the status is NO underflow.
	errs	+= service_ioctl_set_reg_list(fd, list_3);

	if (errs == 0)
	{
		sts	= -1;
		ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AI_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AISS16AO2_BUF_UNDERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. Underflow occurred but was unexpected.)\n",
					__LINE__);
		}
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	ai_buf_underflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AI_BUF_UNDERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_underflow_test(int fd)
{
	int		errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AI_BUF_UNDERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);



}


