// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_buf_over_data.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32	_buffer[64L * 1024];



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	AISS16AO2_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Force a buffer overflow.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int		errs	= 0;
	int		ret;
	s32		val;

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	memset(_buffer, 0xFF, sizeof(_buffer));

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 01) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	val		= -1;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_DATA, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. There is an unexpected overflow.)\n",
				__LINE__);
	}

	errs	+= service_ioctl_set_reg_list(fd, list_2);

	val		= -1;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_DATA, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Overflow expected but did not occur.)\n",
				__LINE__);
	}

	val		= AISS16AO2_BUF_OVERFLOW_TEST;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_DATA, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was unexpectedly cleared.)\n",
				__LINE__);
	}

	val		= AISS16AO2_BUF_OVERFLOW_CLEAR;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_DATA, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was not cleared.)\n",
				__LINE__);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_buf_over_data_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_BUF_OVER_DATA.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_over_data_test(int fd)
{
	int	errs		= 0;

	gsc_label("AISS16AO2_IOCTL_AO_BUF_OVER_DATA");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


