// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_burst_status.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TRIGGER_MODE,
			/* arg		*/	AISS16AO2_TRIGGER_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_C_NDIV,
			/* arg		*/	0xFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_C_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_BURST_ENABLE,
			/* arg		*/	AISS16AO2_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Put data in the output FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0x0000
		},
		{					// Add an EOF entry.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0x10000
		},
		// There are now 11 values in the output buffer.
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		// It appears that one value is pulled for staging.
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the buffer has 10 values.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x0000A
		},
		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// This should empty the buffer very quickly.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Wait a moment.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the buffer is now empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{					// Put data in the output FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0x0000
		},
		// One value gets pulled for staging.
		{					// Verify that the buffer has 9 value.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00009
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	s32	status;

	for (;;)	// A convenience loop.
	{
		errs	= aiss16ao2_initialize(fd, -1, 0);

		if (errs)
		{
			printf("FAIL <---  (initialize @ %d)\n", __LINE__);
			break;
		}

		// Verify READY.
		errs	= service_ioctl_set_reg_list(fd, list_1);

		if (errs)
		{
			printf("FAIL <---  (list_1 @ %d)\n", __LINE__);
			break;
		}

		errs	= aiss16ao2_ao_burst_status(fd, -1, 0, &status);

		if (errs)
		{
			printf("FAIL <---  (burst status @ %d)\n", __LINE__);
			break;
		}

		if (status != AISS16AO2_AO_BURST_STATUS_READY)
		{
			errs	= 1;
			printf("FAIL <---  (Burst status is not READY @ %d)\n", __LINE__);
			break;
		}

		// Verify NOT READY.
		errs	= service_ioctl_set_reg_list(fd, list_2);

		if (errs)
		{
			printf("FAIL <---  (list_2 @ %d)\n", __LINE__);
			break;
		}

		errs	= aiss16ao2_ao_burst_status(fd, -1, 0, &status);

		if (errs)
		{
			printf("FAIL <---  (burst status @ %d)\n", __LINE__);
			break;
		}

		if (status != AISS16AO2_AO_BURST_STATUS_BUSY)
		{
			errs	= 1;
			printf("FAIL <---  (Burst status is not BUSY @ %d)\n", __LINE__);
			break;
		}

		// Verify READY.
		errs	= service_ioctl_set_reg_list(fd, list_3);

		if (errs)
		{
			printf("FAIL <---  (list_3 @ %d)\n", __LINE__);
			break;
		}

		errs	= aiss16ao2_ao_burst_status(fd, -1, 0, &status);

		if (errs)
		{
			printf("FAIL <---  (burst status @ %d)\n", __LINE__);
			break;
		}

		if (status != AISS16AO2_AO_BURST_STATUS_READY)
		{
			errs	= 1;
			printf("FAIL <---  (Burst status is not READY @ %d)\n", __LINE__);
			break;
		}

		// Cleanup
		errs	= aiss16ao2_initialize(fd, -1, 0);

		if (errs)
		{
			printf("FAIL <---  (initialize @ %d)\n", __LINE__);
			break;
		}

		break;
	}

	return(errs);
}




/******************************************************************************
*
*	Function:	ao_burst_status_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_BURST_STATUS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_burst_status_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AO_BURST_STATUS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


