// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/main.h $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <semaphore.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "16aiss16ao2_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index;			// device index
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of boards detected.
	s32	test_limit;		// for continuous testing

	int	fd;				// File descriptor for device to access.

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_REG_WRITE,
	SERVICE_SLEEP,
	SERVICE_SLEEP_MS,
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	unsigned long	cmd;	// IOCTL code
	long			arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************

int	ai_buf_clear_test(int fd);
int	ai_buf_level_test(int fd);
int	ai_buf_overflow_test(int fd);
int	ai_buf_underflow_test(int fd);
int	ai_burst_enable_test(int fd);
int	ai_burst_size_test(int fd);
int	ai_burst_status_test(int fd);
int	ai_chan_sel_test(int fd);
int	ai_clock_mode_test(int fd);
int	ai_enable_test(int fd);
int	ai_fsamp_range_test(int fd);
int	ai_mode_test(int fd);
int	ai_range_a_test(int fd);
int	ai_range_b_test(int fd);
int	ai_sw_clock_test(int fd);
int	ai_sw_trigger_test(int fd);
int	ai_thresh_lvl_test(int fd);
int	ai_thresh_sts_test(int fd);
int	ao_buf_clear_test(int fd);
int	ao_buf_load_req_test(int fd);
int	ao_buf_load_sts_test(int fd);
int	ao_buf_mode_test(int fd);
int	ao_buf_level_test(int fd);
int	ao_buf_over_data_test(int fd);
int	ao_buf_over_fram_test(int fd);
int	ao_burst_enable_test(int fd);
int	ao_burst_status_test(int fd);
int	ao_chan_sel_test(int fd);
int	ao_clock_mode_test(int fd);
int	ao_clock_status_test(int fd);
int	ao_enable_test(int fd);
int	ao_mode_test(int fd);
int	ao_output_mode_test(int fd);
int	ao_range_test(int fd);
int	ao_sw_clock_test(int fd);
int	ao_sw_trigger_test(int fd);
int	ao_thresh_lvl_test(int fd);
int	ao_thresh_sts_test(int fd);
int	auto_cal_sts_test(int fd);
int	auto_calibrate_test(int fd);

int	data_format_test(int fd);
int	dio_dir_out_test(int fd);
int	dio_read_test(int fd);
int	dio_write_test(int fd);

int	gen_a_enable_test(int fd);
int	gen_a_ndiv_test(int fd);
int	gen_b_enable_test(int fd);
int	gen_b_ndiv_test(int fd);
int	gen_c_enable_test(int fd);
int	gen_c_ndiv_test(int fd);

int	initialize_test(int fd);
int	irq_enable_test(int fd);		// AISS16AO2_IOCTL_IRQ_ENABLE

int	master_clock_test(int fd);

int	perform_tests(const args_t* args);

int	query_test(int fd);

int	reg_mod_test(int fd);
int	reg_read_test(int fd);
int	reg_write_test(int fd);
int	rx_io_abort_test(int fd);		// AISS16AO2_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);
int	rx_io_overflow_test(int fd);
int	rx_io_timeout_test(int fd);
int	rx_io_underflow_test(int fd);

int	service_ioctl_reg_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_reg_list(int fd, const service_data_t* list);

int	trigger_mode_test(int fd);
int	tx_io_abort_test(int fd);		// AISS16AO2_IOCTL_TX_IO_ABORT
int	tx_io_mode_test(int fd);
int	tx_io_overflow_test(int fd);
int	tx_io_timeout_test(int fd);

int	wait_cancel_test(int fd);		// AISS16AO2_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);		// AISS16AO2_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);		// AISS16AO2_IOCTL_WAIT_STATUS



#endif
