// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/reg_read.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_read_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_REG_READ.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_read_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
	} list[]	=
	{
		// reg					value
		{ AISS16AO2_GSC_BCR,	0		},
		{ AISS16AO2_GSC_DIOPR,	0		},
		{ AISS16AO2_GSC_AOC0R,	0		},
		{ AISS16AO2_GSC_AOC1R,	0		},
		{ AISS16AO2_GSC_AICR,	0		},
		{ AISS16AO2_GSC_AIBR,	0		},
		{ AISS16AO2_GSC_RGAR,	0		},
		{ AISS16AO2_GSC_RGBR,	0		},
		{ AISS16AO2_GSC_AIBBSR,	0		},
		{ AISS16AO2_GSC_IBSR,	0		},
		{ AISS16AO2_GSC_IBTR,	0		},
		{ AISS16AO2_GSC_PSR,	0		},
		{ AISS16AO2_GSC_ACR,	0		},
		{ AISS16AO2_GSC_AVR,	0		},
		{ AISS16AO2_GSC_BOOR,	0		},
		{ AISS16AO2_GSC_OBTR,	0		},
		{ AISS16AO2_GSC_OBSR,	0		},
		{ AISS16AO2_GSC_AOBR,	0		},
		{ AISS16AO2_GSC_RGCR,	0		},

		{ GSC_PCI_9056_VIDR,	0x10B5	},
		{ GSC_PCI_9056_DIDR,	0x9056	},
		{ GSC_PCI_9056_SVID,	0x10B5	},
		{ GSC_PCI_9056_SID,		0x3243	},

		{ GSC_PLX_9056_VIDR,	0x10B5	},
		{ GSC_PLX_9056_DIDR,	0x9056	}
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	gsc_label("AISS16AO2_IOCTL_REG_READ");

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= 0xDEADBEEF;
		parm.mask	= 0xBEEFDEAD;
		ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.value == 0xDEADBEEF)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, value not changed)\n", __LINE__, i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		if ((list[i].value) && (parm.value != list[i].value))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


