// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/reg_write.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg					value		mask
		{ AISS16AO2_GSC_IBTR,	0x0002AAAA,	0x0003FFFF	},
		{ AISS16AO2_GSC_IBTR,	0x00000005,	0x0003FFFF	},

		{ AISS16AO2_GSC_OBTR,	0x0002AAAA,	0x0003FFFF	},
		{ AISS16AO2_GSC_OBTR,	0x00000005,	0x0003FFFF	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	gsc_label("AISS16AO2_IOCTL_REG_WRITE");

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_WRITE, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


