// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/rx_io_underflow.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	AISS16AO2_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	AISS16AO2_IO_UNDERFLOW_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{				// Force an underflow
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	2,
			/* reg		*/	AISS16AO2_GSC_AIBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got an underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x8,
			/* value	*/	0x8
		},
		{				// Ignore the underflow
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	AISS16AO2_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Set a low timeout period.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{				// Verify that we got an underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x8,
			/* value	*/	0x8
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	AISS16AO2_IO_UNDERFLOW_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	data[1];
	int	errs	= 0;
	int	ret;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		ret	= aiss16ao2_read(fd, data, 4);

		if (ret != 0)
		{
			errs++;
			printf("FAIL <---  (aiss16ao2_read(): returned %d, expected 0)\n", ret);
		}
	}

	errs	+= service_ioctl_set_reg_list(fd, list_2);

	if (errs == 0)
	{
		ret	= aiss16ao2_read(fd, data, 4);

		if (ret >= 0)
		{
			errs++;
			printf("FAIL <---  (aiss16ao2_read(): returned %d, expected < 0\n", ret);
		}
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	return(errs);
}




/******************************************************************************
*
*	Function:	rx_io_underflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_RX_IO_UNDERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_io_underflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_RX_IO_UNDERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


