// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_config_ai.c $
// $Rev: 52934 $
// $Date: 2023-04-20 15:50:26 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_config_ai
*
*	Purpose:
*
*		Configure the given device's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*		master	The master clock frequency, or -1.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss16ao2_config_ai(int fd, int index, int verbose, s32 fsamp, s32 master_clock)
{
	int	errs	= 0;
	s32	ndiv;
	s32	range;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= aiss16ao2_initialize			(fd, index, verbose);

	// All settings must be applied after the initialization call!

	errs	+= aiss16ao2_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= aiss16ao2_rx_io_overflow		(fd, index, verbose, AISS16AO2_IO_OVERFLOW_CHECK,	NULL);
	errs	+= aiss16ao2_rx_io_timeout		(fd, index, verbose, 30,							NULL);
	errs	+= aiss16ao2_rx_io_underflow	(fd, index, verbose, AISS16AO2_IO_UNDERFLOW_CHECK,	NULL);

	errs	+= aiss16ao2_master_clock		(fd, index, verbose, master_clock,					NULL);
	errs	+= aiss16ao2_data_format		(fd, index, verbose, AISS16AO2_DATA_FORMAT_OFF_BIN,	NULL);

	errs	+= aiss16ao2_ai_clock_mode		(fd, index, verbose, AISS16AO2_CLOCK_MODE_OUT,		NULL);
	errs	+= aiss16ao2_ai_mode			(fd, index, verbose, AISS16AO2_AI_MODE_DIFF,		NULL);
	errs	+= aiss16ao2_ai_thresh_lvl		(fd, index, verbose, 200L * 1024,					NULL);
	errs	+= aiss16ao2_ai_burst_enable	(fd, index, verbose, AISS16AO2_BURST_ENABLE_NO,		NULL);
	errs	+= aiss16ao2_ai_chan_sel		(fd, index, verbose, 0xFFFF,						NULL);
	errs	+= aiss16ao2_ai_range_a			(fd, index, verbose, AISS16AO2_RANGE_10V,			NULL);
	errs	+= aiss16ao2_ai_range_b			(fd, index, verbose, AISS16AO2_RANGE_10V,			NULL);

	errs	+= aiss16ao2_trigger_mode		(fd, index, verbose, AISS16AO2_TRIGGER_MODE_OUT,	NULL);
	errs	+= aiss16ao2_fsamp_ai_compute	(fd, index, verbose, fsamp, &ndiv, &range,			NULL);
	errs	+= aiss16ao2_ai_fsamp_range		(fd, index, verbose, range,							NULL);
	errs	+= aiss16ao2_gen_a_ndiv			(fd, index, verbose, ndiv,							NULL);
	errs	+= aiss16ao2_gen_a_enable		(fd, index, verbose, AISS16AO2_GEN_ENABLE_YES,		NULL);
	errs	+= aiss16ao2_gen_b_enable		(fd, index, verbose, AISS16AO2_GEN_ENABLE_NO,		NULL);
	errs	+= aiss16ao2_fsamp_ai_report_all(fd, index, verbose,								NULL);

	// Settings affecting the sample rate must be applied before autocalibration.
	// Refer to the board user manual for clarification.

//	errs	+= aiss16ao2_auto_calibrate		(fd, index, verbose);	// cable/equipment may need to be disconnected

	// Interrupt selection should be applied after autocalibration.
	// If detecting Autocalibration Done, then apply the setting beforehand.
	errs	+= aiss16ao2_irq_enable			(fd, index, verbose, 0,								NULL);

	errs	+= aiss16ao2_ai_enable			(fd, index, verbose, AISS16AO2_AIO_ENABLE_YES,		NULL);
	errs	+= aiss16ao2_ai_buf_clear		(fd, index, verbose);
	errs	+= aiss16ao2_ai_buf_overflow	(fd, index, verbose, AISS16AO2_BUF_OVERFLOW_CLEAR,	NULL);
	errs	+= aiss16ao2_ai_buf_underflow	(fd, index, verbose, AISS16AO2_BUF_UNDERFLOW_CLEAR,	NULL);
	errs	+= aiss16ao2_ai_buf_level		(fd, index, verbose,								NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


