// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_query.c $
// $Rev: 50690 $
// $Date: 2022-03-07 13:04:59 -0600 (Mon, 07 Mar 2022) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS16AO2_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss16ao2_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AISS16AO2_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";		break;
		case AISS16AO2_QUERY_CHANNEL_AI_MAX:	ptr	= "AI Max Channels";		break;
		case AISS16AO2_QUERY_CHANNEL_AI_QTY:	ptr	= "AI Installed Channels";	break;
		case AISS16AO2_QUERY_CHANNEL_AO_MAX:	ptr	= "AO Max Channels";		break;
		case AISS16AO2_QUERY_CHANNEL_AO_QTY:	ptr	= "AO Installed Channels";	break;
		case AISS16AO2_QUERY_COUNT:				ptr	= "Query Options";			break;
		case AISS16AO2_QUERY_DEVICE_TYPE:		ptr	= "Device Type";			break;
		case AISS16AO2_QUERY_DMDMA:				ptr	= "DMDMA Supported";		break;
		case AISS16AO2_QUERY_FGEN_MAX:			ptr	= "Fgen Max";				break;
		case AISS16AO2_QUERY_FGEN_MIN:			ptr	= "Fgen Min";				break;
		case AISS16AO2_QUERY_FIFO_SIZE_RX:		ptr	= "Rx FIFO Size";			break;
		case AISS16AO2_QUERY_FIFO_SIZE_TX:		ptr	= "Tx FIFO Size";			break;
		case AISS16AO2_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";				break;
		case AISS16AO2_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";				break;
		case AISS16AO2_QUERY_INIT_MS:			ptr	= "Init. Duration";			break;
		case AISS16AO2_QUERY_MASTER_CLOCK:		ptr	= "Master Clock";			break;
		case AISS16AO2_QUERY_NDIV_MASK:			ptr	= "Ndiv Mask";				break;
		case AISS16AO2_QUERY_NDIV_MAX:			ptr	= "Ndiv Max";				break;
		case AISS16AO2_QUERY_NDIV_MIN:			ptr	= "Ndiv Min";				break;
		case AISS16AO2_QUERY_RATE_GEN_QTY:		ptr	= "Rate Gen Qty";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AISS16AO2_QUERY_DMDMA:

			strcpy(buf, set ? "Supported" : "Not Supported");
			break;

		case AISS16AO2_QUERY_CHANNEL_AI_MAX:
		case AISS16AO2_QUERY_CHANNEL_AI_QTY:
		case AISS16AO2_QUERY_CHANNEL_AO_MAX:
		case AISS16AO2_QUERY_CHANNEL_AO_QTY:
		case AISS16AO2_QUERY_COUNT:
		case AISS16AO2_QUERY_NDIV_MAX:
		case AISS16AO2_QUERY_NDIV_MIN:
		case AISS16AO2_QUERY_RATE_GEN_QTY:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AISS16AO2_QUERY_NDIV_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case AISS16AO2_QUERY_FGEN_MAX:
		case AISS16AO2_QUERY_FGEN_MIN:
		case AISS16AO2_QUERY_MASTER_CLOCK:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AISS16AO2_QUERY_AUTO_CAL_MS:
		case AISS16AO2_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AISS16AO2_QUERY_FSAMP_MAX:
		case AISS16AO2_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AISS16AO2_QUERY_FIFO_SIZE_RX:
		case AISS16AO2_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "M";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Words", (long) tmp, ptr);
			break;

		case AISS16AO2_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AISS16AO2)
			{
				strcpy(buf, "16AISS16AO2");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


