// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_reg.c $
// $Rev: 52934 $
// $Date: 2023-04-20 15:50:26 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)		"GSC " #a, AISS16AO2_GSC_##a, 0, 0
#define	WIDTH			22
#define	SIZEOF_ARRAY(a)	(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	aim[]	=
	{
		"Differential",
		"Single Ended",
		"Zero Test",
		"+Vref Test",
		"Output Channel 0",
		"Output Channel 1",
		"RESERVED",
		"RESERVED"
	};

	static const char*	range[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"RESERVED"
	};

	static const char*	disabled[]	= { "Disabled",			"Enabled"		};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	immed[]		= { "Immediate",		"Simultaneous"	};
	static const char*	input[]		= { "Input",			"Output"		};
	static const char*	no[]		= { "No",				"Yes"			};
	static const char*	reg[]		= { "No (registered)",	"Yes (FIFO)"	};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 31, 1, idle,		"Initialize"			);
	gsc_reg_field_show(width, WIDTH, value, 30, 30, 1, disabled,	"AI Warp Mode"			);
	gsc_reg_field_show(width, WIDTH, value, 29, 29, 1, no,			"Auto-Cal Pass"			);
	gsc_reg_field_show(width, WIDTH, value, 28, 28, 1, idle,		"Auto-Cal"				);
	gsc_reg_field_show(width, WIDTH, value, 27, 27, 1, disabled,	"Rate-B Generator"		);
	gsc_reg_field_show(width, WIDTH, value, 26, 26, 1, disabled,	"Rate-A Generator"		);
	gsc_reg_field_show(width, WIDTH, value, 25, 25, 1, twos,		"Data Format"			);
	gsc_reg_field_show(width, WIDTH, value, 24, 24, 1, input,		"AI Clock Initiator"	);
	gsc_reg_field_show(width, WIDTH, value, 23, 23, 1, idle,		"Output SW Trigger"		);
	gsc_reg_field_show(width, WIDTH, value, 22, 22, 1, disabled,	"Rate-C Generator"		);
	gsc_reg_field_show(width, WIDTH, value, 21, 21, 1, input,		"Trigger Initiator"		);
	gsc_reg_field_show(width, WIDTH, value, 20, 20, 1, idle,		"Output SW Clock"		);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, reg,			"Buffered Outputs"		);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, immed,		"Simul. Outputs"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 16, 1, range,		"Output Range"			);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, no,			"Input Overflow"		);
	gsc_reg_field_show(width, WIDTH, value, 14, 14, 1, idle,		"AI Threshold Flag"		);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, idle,		"Clear Input Buffer"	);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, disabled,	"Enable Input Buffer"	);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, idle,		"Input SW Trigger"		);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, idle,		"Input Burst Busy"		);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, disabled,	"Enable Input Burst"	);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, idle,		"Input SW Clock"		);
	gsc_reg_field_show(width, WIDTH, value,  7,  6, 1, range,		"Input Range B"			);
	gsc_reg_field_show(width, WIDTH, value,  5,  4, 1, range,		"Input Range A"			);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, no,			"Input Underflow"		);
	gsc_reg_field_show(width, WIDTH, value,  2,  0, 1, aim,			"Analog Input Mode"		);

	return(0);
}



//*****************************************************************************
static int _diopr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	input[]		= { "Input",	"Output"	};
	static const char*	low[]		= { "0 (low)",	"1 (high)"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 14, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, input,	"DIO 5 Direction"	);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, input,	"DIO 4 Direction"	);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, input,	"DIO 3 Direction"	);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, input,	"DIO 2 Direction"	);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, input,	"DIO 1 Direction"	);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, input,	"DIO 0 Direction"	);
	gsc_reg_field_show(width, WIDTH, value,  7,  6, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value,  5,  5, 1, low,		"DIO 5 Data"		);
	gsc_reg_field_show(width, WIDTH, value,  4,  4, 1, low,		"DIO 4 Data"		);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, low,		"DIO 3 Data"		);
	gsc_reg_field_show(width, WIDTH, value,  2,  2, 1, low,		"DIO 2 Data"		);
	gsc_reg_field_show(width, WIDTH, value,  1,  1, 1, low,		"DIO 1 Data"		);
	gsc_reg_field_show(width, WIDTH, value,  0,  0, 1, low,		"DIO 0 Data"		);

	return(0);
}



//*****************************************************************************
static int _aicr_decode(int fd, int supported, u32 value, int width)
{
	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 16, 1, NULL, "Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 15,  0, 1, NULL, "Active Channels"	);

	return(0);
}



//*****************************************************************************
static int _aibr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	no[]	= { "No",	"Yes"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 18, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, no,		"End of Burst"	);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, no,		"First Channel"	);
	gsc_reg_field_show(width, WIDTH, value, 15,  0, 1, NULL,	"Data"			);

	return(0);
}



//*****************************************************************************
static int _rgxr_decode(int fd, int supported, u32 value, int width)
{
	u32	field;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 24, 1, NULL,	"Reserved"	);

	gsc_reg_field_show(width, WIDTH, value, 23,  0, 0, NULL,	"Ndiv"		);
	field	= value & 0x00FFFFFF;
	printf(" Fclk / %ld\n", (long) field);

	return(0);
}



//*****************************************************************************
static int _aibbsr_decode(int fd, int supported, u32 value, int width)
{
	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 24, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 23,  0, 1, NULL,	"Burst Block Size"	);

	return(0);
}



//*****************************************************************************
static int _xbsr_decode(int fd, int supported, u32 value, int width)
{
	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 19, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, WIDTH, value, 18,  0, 1, NULL,	"Fill Level"	);

	return(0);
}



//*****************************************************************************
static int _xbtr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]	= { "Clear",	"Set"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, clear,	"Threshold Flag"	);
	gsc_reg_field_show(width, WIDTH, value, 18,  0, 1, NULL,	"Threshold Level"	);

	return(0);
}



//*****************************************************************************
static int _psr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]		= { "Idle",		"Active"	};
	static const char*	ignore[]	= { "Ignore",	"Selected"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 31, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 30, 30, 1, idle,	"Output Overflow"		);
	gsc_reg_field_show(width, WIDTH, value, 29, 29, 1, idle,	"Output Burst Ready"	);
	gsc_reg_field_show(width, WIDTH, value, 28, 28, 1, idle,	"Load Ready Low->High"	);
	gsc_reg_field_show(width, WIDTH, value, 27, 27, 1, idle,	"Load Ready High->Low"	);
	gsc_reg_field_show(width, WIDTH, value, 26, 26, 1, idle,	"AO Thresh. Low->High"	);
	gsc_reg_field_show(width, WIDTH, value, 25, 25, 1, idle,	"AO Thresh. High->Low"	);
	gsc_reg_field_show(width, WIDTH, value, 24, 24, 1, idle,	"DIO 0 Low->High"		);
	gsc_reg_field_show(width, WIDTH, value, 23, 23, 1, idle,	"Analog Output Clock"	);
	gsc_reg_field_show(width, WIDTH, value, 22, 22, 1, idle,	"Analog Input Clock"	);
	gsc_reg_field_show(width, WIDTH, value, 21, 21, 1, idle,	"AI Burst Completed"	);
	gsc_reg_field_show(width, WIDTH, value, 20, 20, 1, idle,	"AI Burst Initiated"	);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, idle,	"Input Over/Underflow"	);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, idle,	"AI Thresh. Low->High"	);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, idle,	"AI Thresh. High->Low"	);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, idle,	"Auto Cal Done"			);

	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 14, 14, 1, ignore,	"Output Overflow"		);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, ignore,	"Output Burst Ready"	);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, ignore,	"Load Ready Low->High"	);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, ignore,	"Load Ready High->Low"	);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, ignore,	"AO Thresh. Low->High"	);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, ignore,	"AO Thresh. High->Low"	);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, ignore,	"DIO 0 Low->High"		);
	gsc_reg_field_show(width, WIDTH, value,  7,  7, 1, ignore,	"Analog Output Clock"	);
	gsc_reg_field_show(width, WIDTH, value,  6,  6, 1, ignore,	"Analog Input Clock"	);
	gsc_reg_field_show(width, WIDTH, value,  5,  5, 1, ignore,	"AI Burst Completed"	);
	gsc_reg_field_show(width, WIDTH, value,  4,  4, 1, ignore,	"AI Burst Initiated"	);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, ignore,	"Input Over/Underflow"	);
	gsc_reg_field_show(width, WIDTH, value,  2,  2, 1, ignore,	"AI Thresh. Low->High"	);
	gsc_reg_field_show(width, WIDTH, value,  1,  1, 1, ignore,	"AI Thresh. High->Low"	);
	gsc_reg_field_show(width, WIDTH, value,  0,  0, 1, ignore,	"Auto Cal Done"			);

	return(0);
}



//*****************************************************************************
static int _acr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	master[]	=
	{
		"45 MHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_16[]	= { "16 Channels",	"8 Channels"	};
	static const char*	_2[]	= { "2 Channels",	"0 Channels"	};

	u32	field;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 19, 18, 1, master,	"Master Clock"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, _2,		"Output Channels"	);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, _16,		"Input Channels"	);
	gsc_reg_field_show(width, WIDTH, value, 15, 12, 1, NULL,	"Reserved"			);

	gsc_reg_field_show(width, WIDTH, value, 11,  0, 0, NULL,	"Firmware Revision"	);
	field	= value & 0xFFF;
	printf("%03lX\n", (long) field);

	return(0);
}



//*****************************************************************************
static int _boor_decode(int fd, int supported, u32 value, int width)
{
	static const char*	disable[]	= { "Disabled",	"Enabled"			};
	static const char*	ext[]		= { "External",	"Rate-C Generator"	};
	static const char*	idle[]		= { "Idle",		"Active"			};
	static const char*	no[]		= { "No",		"Yes"				};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 22, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 21, 21, 1, no,		"DMDMA Chan. Swap"		);
	gsc_reg_field_show(width, WIDTH, value, 20, 20, 1, idle,	"Output SW Trigger"		);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, no,		"Output Burst Enable"	);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, no,		"Burst Ready"			);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, no,		"AO Frame Overflow"		);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, no,		"AO Buffer Overflow"	);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, no,		"AO Buffer Full"		);
	gsc_reg_field_show(width, WIDTH, value, 14, 14, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, idle,	"Buffer Thresh. Flag"	);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, no,		"AO Buffer Empty"		);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, idle,	"AO Buffer Clear"		);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, no,		"Load Ready"			);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, idle,	"Load Request"			);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, no,		"Circular Buffer"		);
	gsc_reg_field_show(width, WIDTH, value,  7,  7, 1, idle,	"Output SW Clock"		);
	gsc_reg_field_show(width, WIDTH, value,  6,  6, 1, no,		"Output Clock Ready"	);
	gsc_reg_field_show(width, WIDTH, value,  5,  5, 1, disable,	"Output Clocking"		);
	gsc_reg_field_show(width, WIDTH, value,  4,  4, 1, ext,		"Output Clock Source"	);
	gsc_reg_field_show(width, WIDTH, value,  3,  2, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value,  1,  1, 1, no,		"Output Chan 1 Active"	);
	gsc_reg_field_show(width, WIDTH, value,  0,  0, 1, no,		"Output Chan 0 Active"	);

	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCR),		0,	_bcr_decode,	"Board Control Register"				},
	{ _GSC_REG(DIOPR),		0,	_diopr_decode,	"Digital I/O Port Register"				},
	{ _GSC_REG(AOC0R),		0,	NULL,			"Anlog Output Channel 0 Register"		},
	{ _GSC_REG(AOC1R),		0,	NULL,			"Anlog Output Channel 1 Register"		},
	{ _GSC_REG(AICR),		0,	_aicr_decode,	"Active Input Channels Register"		},
	{ _GSC_REG(AIBR),		0,	_aibr_decode,	"Analog Input Buffer Register"			},
	{ _GSC_REG(RGAR),		0,	_rgxr_decode,	"Rate Generator A Register"				},
	{ _GSC_REG(RGBR),		0,	_rgxr_decode,	"Rate Generator B Register"				},
	{ _GSC_REG(AIBBSR),		0,	_aibbsr_decode,	"Analog Input Burst Block Size Register"},
	{ _GSC_REG(IBSR),		0,	_xbsr_decode,	"Input Buffer Size Register"			},
	{ _GSC_REG(IBTR),		0,	_xbtr_decode,	"Input Buffer Threshold Register"		},
	{ _GSC_REG(PSR),		0,	_psr_decode,	"Primary Status Register"				},
	{ _GSC_REG(ACR),		0,	_acr_decode,	"Assymbly Configuration Register"		},
	{ _GSC_REG(AVR),		0,	NULL,			"Auto Cal Values Register"				},
	{ _GSC_REG(BOOR),		0,	_boor_decode,	"Buffered Output Operations Register"	},
	{ _GSC_REG(OBTR),		0,	_xbtr_decode,	"Output Buffer Threshold Register"		},
	{ _GSC_REG(OBSR),		0,	_xbsr_decode,	"Output Buffer Size Register"			},
	{ _GSC_REG(AOBR),		0,	NULL,			"Analog Output Buffer Register"			},
	{ _GSC_REG(RGCR),		0,	_rgxr_decode,	"Rate Generator C Register"				},

	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	aiss16ao2_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aiss16ao2_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	aiss16ao2_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aiss16ao2_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	aiss16ao2_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aiss16ao2_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	aiss16ao2_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aiss16ao2_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	aiss16ao2_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss16ao2_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, aiss16ao2_reg_read);
	return(errs);
}



